package com.i2edu.flutter_ali_camera

import android.content.Context
import android.graphics.Bitmap
import android.net.Uri
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.view.*
import android.widget.FrameLayout
import com.aliyun.editor.EditorCallBack
import com.aliyun.qupai.editor.AliyunIComposeCallBack
import com.aliyun.qupai.editor.impl.AliyunEditorFactory
import com.aliyun.qupai.import_core.AliyunImportCreator
import com.aliyun.recorder.AliyunRecorderCreator
import com.aliyun.recorder.supply.AliyunIRecorder
import com.aliyun.recorder.supply.RecordCallback
import com.aliyun.svideo.sdk.external.struct.common.AliyunVideoClip
import com.aliyun.svideo.sdk.external.struct.common.AliyunVideoParam
import com.aliyun.svideo.sdk.external.struct.common.VideoDisplayMode
import com.aliyun.svideo.sdk.external.struct.common.VideoQuality
import com.aliyun.svideo.sdk.external.struct.effect.EffectBean
import com.aliyun.svideo.sdk.external.struct.effect.EffectFilter
import com.aliyun.svideo.sdk.external.struct.encoder.VideoCodecs
import com.aliyun.svideo.sdk.external.struct.recorder.CameraParam
import com.aliyun.svideo.sdk.external.struct.recorder.CameraType
import com.aliyun.svideo.sdk.external.struct.recorder.MediaInfo
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.platform.PlatformView
import java.io.File
import java.lang.Exception

class AliCameraView(private val context: Context, private val binaryMessenger: BinaryMessenger, id: Int, param: Any?) : PlatformView, MethodChannel.MethodCallHandler {
    private val TAG = "AliCameraView"
    var surfaceView: SurfaceView? = null
    var frameLayout: FrameLayout? = null

    private var methodChannel: MethodChannel? = null
    private var recorder: AliyunIRecorder? = null
    private val mainHandler: Handler = Handler(Looper.getMainLooper())

    init {
        frameLayout = FrameLayout(context)
        frameLayout?.layoutParams = ViewGroup.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        surfaceView = SurfaceView(context)
        surfaceView?.layoutParams = ViewGroup.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        frameLayout?.addView(surfaceView)

        methodChannel = MethodChannel(binaryMessenger, "com.i2edu.cameraLib/camera_view_$id")
        methodChannel?.setMethodCallHandler(this)
    }

    override fun getView(): View {
        return frameLayout!!
    }

    override fun dispose() {
        if (frameLayout != null) {
            frameLayout?.removeView(surfaceView)
            frameLayout = null
        }
        if (surfaceView != null) {
            surfaceView = null
        }
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "create" -> {
                val recordOption: Map<String, Any> = call.argument<Map<String, Any>>("recordOption")!!
                setUpCameraView(recordOption = recordOption)
                result.success(true)
            }
            "startPreview" -> {
                startPreview()
                result.success(true)
            }
            "stopPreview" -> {
                stopPreview()
                result.success(true)
            }
            "switchCamera" -> {
                switchCamera()
                result.success(true)
            }
            "setBeauty" -> {
                setBeauty(call.argument<Int>("level")!!)
                result.success(true)
            }
            "setFilter" -> {
                setFilter(call.argument<String>("path")!!)
                result.success(true)
            }
            "startRecord" -> {
                startRecord(
                        max = call.argument<Int>("max")!!, recordPath = call.argument<String>("recordPath")!!, result = result)
            }
            "startCompose" -> {
                val outputPath = call.argument<String>("outputPath")!!
                val bgmPath = call.argument<String>("bgmPath")
                val paths = call.argument<List<String>>("paths")!!
                val durations = call.argument<List<Int>>("durations")!!
                val outputOption = call.argument<Map<String, Any>>("composeOption")!!
                startCompose(outputPath = outputPath, bgmPath = bgmPath, paths = paths, durations = durations, outputOption = outputOption, result = result)
            }
            "onDestroy" -> {
                onDestroy()
                result.success(true)
            }
            else -> result.notImplemented()
        }
    }

    private fun setUpCameraView(recordOption: Map<String, Any>) {
        try {
            recorder = AliyunRecorderCreator.getRecorderInstance(context)

            val mediaInfo = MediaInfo()
            if (recordOption.containsKey("videoWidth")) mediaInfo.videoWidth = recordOption["videoWidth"] as Int
            if (recordOption.containsKey("videoHeight")) mediaInfo.videoHeight = recordOption["videoHeight"] as Int
            if (recordOption.containsKey("videoCodecs")) mediaInfo.videoCodec = VideoCodecs.getInstanceByValue(recordOption["videoCodecs"] as Int)
            if (recordOption.containsKey("fps")) mediaInfo.fps = recordOption["fps"] as Int
            if (recordOption.containsKey("crf")) mediaInfo.crf = recordOption["crf"] as Int
            if (recordOption.containsKey("encoderFps")) mediaInfo.encoderFps = recordOption["encoderFps"] as Int
            recorder?.setMediaInfo(mediaInfo)
            if (recordOption.containsKey("quality")) recorder?.setVideoQuality(VideoQuality.values()[recordOption["quality"] as Int])
            if (recordOption.containsKey("videoBitrate")) recorder?.setVideoBitrate(recordOption["videoBitrate"] as Int)
            if (recordOption.containsKey("gop")) recorder?.setGop(recordOption["gop"] as Int)

            recorder?.setFocusMode(CameraParam.FOCUS_MODE_CONTINUE)
            recorder?.setCamera(CameraType.FRONT)
            recorder?.setBeautyStatus(false)
            recorder?.beautyLevel = 0

            recorder?.setRecordCallBack(object : RecordCallback {
                override fun onFinish(p0: String?) {
                }

                override fun onInitReady() {
                }

                override fun onPictureBack(p0: Bitmap?) {
                }

                override fun onPictureDataBack(p0: ByteArray?) {
                }

                override fun onComplete(p0: Boolean, p1: Long) {
                    mainHandler.post {
                        methodChannel?.invokeMethod("recordUpdate",
                                mapOf("progress" to recorder?.clipManager?.maxDuration, "path" to recorder?.clipManager?.videoPathList?.last()))
                    }
                }

                override fun onMaxDuration() {
                }

                override fun onDrawReady() {
                }

                override fun onProgress(p0: Long) {
                    mainHandler.post {
                        methodChannel?.invokeMethod("recordUpdate", mapOf("progress" to p0))
                    }
                }

                override fun onError(p0: Int) {
                }
            })
            recorder?.setDisplayView(surfaceView)
        } catch (e: Exception) {
            Log.e(TAG, "set up camera error $e")
        }
    }

    private fun startPreview() {
        recorder?.startPreview()
    }

    private fun stopPreview() {
        recorder?.stopPreview()
    }

    private fun switchCamera() {
        recorder?.switchCamera()
    }

    private fun setBeauty(level: Int) {
        recorder?.setBeautyStatus(level != 0)
        recorder?.beautyLevel = level
    }

    private fun setFilter(path: String) {
        if (path.isEmpty()) {
            recorder?.applyFilter(EffectFilter(null))
            return
        }
        recorder?.applyFilter(EffectFilter(path))
    }

    private fun startRecord(max: Int, recordPath: String, result: MethodChannel.Result) {
        try {
            recorder?.setOutputPath(recordPath)
            //  补充时间段上去
            val currentDuration = if (recorder?.clipManager?.maxDuration!! >= Int.MAX_VALUE) 0 else recorder?.clipManager?.maxDuration!!
            recorder?.clipManager?.maxDuration = max + currentDuration
            recorder?.startRecording()
            result.success(true)
        } catch (e: Exception) {
            result.error("1001", e.message, null)
        }
    }

    private fun startCompose(outputPath: String, bgmPath: String?, paths: List<String>, durations: List<Int>, outputOption: Map<String, Any>, result: MethodChannel.Result) {
        try {
            // 配置输出参数
            val mImport = AliyunImportCreator.getImportInstance(context)
            for ((index, path) in paths.withIndex()) {
                mImport.addMediaClip(AliyunVideoClip.Builder()
                        .source(path)
                        .startTime(0)
                        .endTime(durations[index].toLong())
                        .build())
            }
            val param = AliyunVideoParam()
            if (outputOption.containsKey("outputWidth")) param.outputWidth = outputOption["outputWidth"] as Int
            if (outputOption.containsKey("outputHeight")) param.outputHeight = outputOption["outputHeight"] as Int
            if (outputOption.containsKey("frameRate")) param.frameRate = outputOption["frameRate"] as Int
            if (outputOption.containsKey("crf")) param.crf = outputOption["crf"] as Int
            if (outputOption.containsKey("gop")) param.gop = outputOption["gop"] as Int
            if (outputOption.containsKey("bitrate")) param.bitrate = outputOption["bitrate"] as Int
            if (outputOption.containsKey("scaleRate")) param.scaleRate = outputOption["scaleRate"] as Float
            if (outputOption.containsKey("videoCodecs")) param.videoCodec = VideoCodecs.getInstanceByValue(outputOption["videoCodecs"] as Int)
            if (outputOption.containsKey("quality")) param.videoQuality = VideoQuality.values()[outputOption["quality"] as Int]
            if (outputOption.containsKey("scaleMode")) param.scaleMode = VideoDisplayMode.values()[outputOption["scaleMode"] as Int]
            mImport.setVideoParam(param)
            val configStringPath = mImport.generateProjectConfigure()
            mImport.release()
            val uri = Uri.fromFile(File(configStringPath))
            // 添加bgm背景音乐
            val editor = AliyunEditorFactory.creatAliyunEditor(uri, object : EditorCallBack() {
                override fun onTextureRender(srcTextureID: Int, width: Int, height: Int): Int {
                    return srcTextureID
                }

                override fun onPlayProgress(p0: Long, p1: Long) {
                }

                override fun onCustomRender(srcTextureID: Int, width: Int, height: Int): Int {
                    return srcTextureID
                }

                override fun onDataReady() {
                }

                override fun onEnd(state: Int) {
                }

                override fun onError(errorCode: Int) {
                }
            })
            editor.init(null, frameLayout?.context)
            if (bgmPath != null) {
                val bean = EffectBean()
                bean.path = bgmPath
                editor.applyMusic(bean)
            }
            // 合并输出视频
            editor.compose(param, outputPath, object : AliyunIComposeCallBack {
                override fun onComposeProgress(p0: Int) {
                    Log.d(TAG, "onComposeProgress: $p0")
                }

                override fun onComposeCompleted() {
                    Log.d(TAG, "onComposeCompleted")
                    mainHandler.post {
                        result.success(true)
                    }
                }

                override fun onComposeError(p0: Int) {
                    Log.d(TAG, "onComposeError: $p0")
                    mainHandler.post {
                        result.success(false)
                    }
                }
            })
        } catch (e: Exception) {
            e.printStackTrace()
            result.error("1002", e.message, null)
        }
    }

    private fun onDestroy() {
        if (recorder != null) {
            recorder?.stopPreview()
            recorder?.stopRecording()
            recorder?.destroy()
            recorder = null
        }
    }
}